# Программа обучения пользователя работе с `TessaClient` { #user-guide-app }

Здесь описано назначение и использование каркасного проекта программы обучения
пользователя работе с `TessaClient` с использованием возможностей автоматизации.

!!! note
Для работы с проектом необходим `.NET 6.0`.

Представим его основные возможности:

- различные виды окон:
    - главное окно управления обучением с кнопками `Меню` и `Выход`;
    - информационное окно без указания элемента;  
    - окно с вопросом к пользователю;  
    - окно подсветки элементов пользовательского интерфейса;  
    - информационное окно с указанием элемента;  
    - окно выбора сценария обучения;  
    - окно повторения последней операции;  
- автопостроение меню с выбором сценариев обучения исходя из сценариев, присутствующих в проекте;
- различные варианты запуска сценариев:
    - до основного цикла обучения;
    - сценарий обучения;
    - после основного цикла обучения;
    - сценарий, исключенный из меню, но который можно программно запустить из другого сценария;
- гибкие механизмы отслеживания выполнения действий с возможностью повторения неудачного действия;
- гибкие механизмы построения окон с подсказками пользователю.

## Краткая инструкция по работе с проектом { #user-guide-app-intro }

Для работы вам потребуется скачать [`Winium.Desktop.Driver` версии не ниже 2.1.1][winiumdriver] и распаковать архив.

## Настройка для запуска { #user-guide-app-configuration }

Все настройки программы обучения пользователя содержатся в файле `app.json`.

| Настройка             | Описание                                                                                                                                                                                                                                | Пример                                                                        |
| --------------------- |-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------| ----------------------------------------------------------------------------- |
| `DriverPath`          | Полный путь к драйверу `Winium`, включая имя файла. Заполняется только если нужно запускать драйвер. При подключении к уже запущенному драйверу заполнять не требуется. Если вы не уверены, запущен драйвер или нет, заполните это поле | `"DriverPath": "..\\Winium.Desktop.Driver.2.1.1\\Winium.Desktop.Driver.exe",` |
| `ClientPath`          | Полный путь к `TessaClient`, включая имя файла. Не заполняется только если требуется подключиться к уже запущенному `TessaClient`                                                                                                       | `"ClientPath": "..\\..\\TessaClient.exe",`                                    |
| `UserName`            | Логин пользователя. Заполняется, если необходимо запустить `TessaClient` для указанного пользователя. Связан с `ClientPath`                                                                                                             | `"UserName": "1111",`                                                         |
| `Password`            | Пароль пользователя. Заполняется, если необходимо запустить `TessaClient` для указанного пользователя. Если совпадает с `UserName`, то может не заполняться. Связан с `ClientPath` и `UserName`                                         | `"Password": "1111",`                                                         |
| `GeneralDelay`        | Базовая задержка действий автоматизации в формате времени (по умолчанию 0.5 секунд). Меняя данную настройку, вы можете ускорять или замедлять действия, выполняемые автоматически                                                       | `"GeneralDelay": "00:00:00.5",`                                               |
| `DialogDelay`         | Базовая задержка отображения окна с подсказкой пользователю в формате времени (по умолчанию 1 секунда). Меняя данную настройку, вы можете ускорять или замедлять время отображения подсказок пользователю                               | `"DialogDelay": "00:00:01",`                                                  |
| `MaxInteractionDelay` | Максимальное время ожидания действия пользователя в формате времени (по умолчанию 5 минут). Меняя данную настройку, вы можете управлять временем ожидания выполнения пользователем требуемых действий                                   | `"MaxInteractionDelay": "00:05:00"`                                           |

!!! note
    Все указанные здесь задержки (`GeneralDelay`, `DialogDelay`, `MaxInteractionDelay`)
    являются базовыми. В программе задаются коэффициенты (множители) для них.
    Поэтому изменяя значения здесь, можно повлиять на воспроизведение всего
    сценария.

Для некоторых параметров файла `app.json` определены соответствующие параметры командной строки.  
Для всех аргументов используется стандартный, принятый в `TESSA`, синтаксис `<Ключ>:<Значение>`.  
Возможно также использовать синтаксис `<Ключ>=<Значение>`.

| Настройка    | Аргумент командной строки |
| ------------ | ------------------------- |
| `DriverPath` | `-d`                      |
| `ClientPath` | `-c`                      |
| `UserName`   | `-u`                      |
| `Password`   | `-p`                      |

## Структура проекта и разработка сценариев { #user-guide-app-guide-development }

Все сценарии должны располагаться в директории `Guides` и реализовывать интерфейс `IGuide`.
Рекомендуется наследоваться от абстрактного базового класса `Guide`,
который содержит базовую реализацию некоторых деталей интерфейса `IGuide`.
Примеры сценариев вы также найдёте в указанной директории.

Рекомендуем при написании сценариев не менять никакой код снаружи директории `Guides`.
При необходимости создавайте статические классы помощники и другие
вспомогательные файлы в директории `Guides` и её дочерних директориях.

!!! note
    Всё, что находится вне директории `Guides`, считается инфраструктурными компонентами
    и может быть заменено в следующей версии каркасного проекта. Чтобы защитить
    свои наработки от изменений при обновлении, располагайте их в *пользовательской*
    директории `Giudes`.

Рассмотрим содержимое проекта по директориям.

### Корневая директория { #user-guide-app-root-dir }

В корневой директории содержатся классы общего назначения.

| Класс                   | Описание                                                                                                                                                                                                                                                                            |
| ----------------------- | ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| `IGuide`                | Интерфейс, который необходимо реализовать в каждом сценарии. В противном случае ваш сценарий не будет запущен. Свойство `Order` задаёт порядок выполнения сценария и размещения его в меню. Не забудьте задать правильный порядок! Свойство `Caption` определяет имя пункта в меню |
| `Guide`                 | Базовый абстрактный класс для всех сценариев. Наследуйте свои сценарии от него                                                                                                                                                                                                     |
| `GuideState`            | Перечисление с основными вариантами запуска сценариев. Каждый сценарий должен указывать требуемый ему вариант запуска |
| `GuideHelper`           | Основной класс помощников для пользовательских сценариев                                                                                                                                                                                                                           |
| `GuideProvider`         | Провайдер для запуска сценариев. Отвечает за всю инфраструктурную информацию. Запускает драйвер и клиент, подключается к драйверу и находит главное окно клиента. Содержит информацию по задержкам операций                                                                        |
| `GuideDiscardException` | Ожидаемая операция не выполнена пользователем                                                                                                                                                                                                                                      |
| `DispatcherHelper`      | Статический помощник по работе с `UI` потоком `WPF`. Вся работа с этим классом скрыта от файлов сценариев                                                                                                                                                                          |
| `Program`               | Главный класс приложения. Не модифицируйте его                                                                                                                                                                                                                                     |
| `Options`               | Содержит считанные из файла `app.json` настройки приложения. Вам не требуется работать с этим классом                                                                                                                                                                              |

### Директория `Extensions` { #user-guide-app-extensions-dir }

Содержит классы расширений.

| Класс                        | Описание                                                                                                                                               |
| ---------------------------- | ------------------------------------------------------------------------------------------------------------------------------------------------------ |
| `GuideProviderExtensions`    | Расширения для класса `GuideProvider`. Если нужно добавить новую функциональность для провайдера, размещайте её в одноимённом классе в папке `Guides` |
| `WebElementExtensions`       | Расширения для класса `WebElement`. Если нужно добавить новую функциональность для `IWebElement`, размещайте её в одноимённом классе в папке `Guides` |
| `SystemParametersExtensions` | Расширения для класса `SystemParameters`. Класс содержит методы и свойства по работе с разрешением и размерами рабочих столов |

### Директория `Windows` { #user-guide-app-windows-dir }

Содержит классы окон и классы помощники работы с ними. Далее описаны только те классы, которые вам нужно будет использовать в сценариях. Остальные классы являются служебными.

| Класс                   | Описание                                                                                                                                                                                                                                                                                                                                   |
| ----------------------- | ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------ |
| `Highlighter`           | Помощник по отображению окна визуального выделения (подсветки) элемента или прямоугольной области. Одновременно может подсветить одну и более областей. Обратите внимание, что окно с подсветкой отображается поверх элемента, поэтому элемент будет терять фокус! |
| `Placement`             | Перечисление с основными положениями окна относительно элемента |
| `ArrowPlacement`        | Перечисление для определения положения указывающей стрелки в дополнении к базовому положению|
| `ButtonDescriptor`      | Запись, описывающая кнопку для отображения в окне с подсказкой в режиме диалога |
| `TooltipOptionsBuilder` | Помощник по построению настроек отображения окна с подсказкой. Рекомендуется использовать этот построитель везде, где необходимо предоставить настройки отображения окна с подсказкой. Вручную создавать класс обычно не требуется, т.к. методы `ShowTooltipAsync` передают лямбда-выражение для конфигурирования экземпляра этого класса |
| `Tooltip`               | Помощник по отображению окна подсказки |
| `StandardButtons`       | Помощник, содержащий стандартные кнопки ("Готово", "Да" и "Нет"). Размещайте дескрипторы собственных кнопок в одноимённом классе в папке `Guides` |

### Директория `Guides` { #user-guide-app-guides-dir }

Содержит классы сценариев. Размещайте классы сценариев и любые вспомогательные классы здесь.

| Класс                   | Описание                                                      |
| ----------------------- | ------------------------------------------------------------- |
| `IntroGuide`            | Сценарий приветствия с обучением по типам подсказок           |
| `ClientWindowGuide`     | Сценарий обучения работы с окном `TessaClient`                |
| `BackgroundChangeGuide` | Сценарий обучения по смене фона                               |
| `MyPreferencesGuide`    | Сценарий обучения работе с "Моими настройками"                |
| `ConclusionGuide`       | Заключительный сценарий. Вызывается после завершения обучения |

## Сборка проекта для запуска на компьютере конечного пользователя { #user-guide-app-build }

Данная программа предназначена для работы на компьютере конечного пользователя.  
Для работы программы вне зависимости от наличия на компьютере конечного пользователя
`.NET 6.0`, необходимо собрать программу с настройками,
копирующими необходимые компоненты среды времени выполнения.

Обратите внимание, что нужно собирать программу для архитектур `x64` и `x86`.

```ps1
dotnet publish -c Release -r win-x64 --sc -o bin\publish\x64
dotnet publish -c Release -r win-x86 --sc -o bin\publish\x86
```

## Краткое типовое руководство пользователя { #user-guide-app-template-launch-guide }

Для работы с программой обучения необходимо выполнить следующие шаги.  
Обратите внимание, что первые два шага нужно выполнять только при первом развёртывании или
обновлении программы обучения. Для повторного запуска уже развёрнутой программы обучения
необходимо начинать с шага 3 представленного ниже алгоритма.

1. Скачать и распаковать архив `cs-user-guide` (выполняется разово при первой установке или при обновлении).
1. Скачать и распаковать архив [`Winium.Desktop.Driver`][winiumdriver] (выполняется разово при первой установке или при обновлении).
1. Запустить `Winium.Desktop.Driver.exe` из папки в п.2.
1. Через `Tessa Applications` запустить `TessaClient`. Минимальная поддерживаемая версия `TESSA 3.6.0.9`.
1. Запустить `cs-user-guide.exe` из папки в п.1.
1. Пройти сценарий, по завершению нажать кнопку `Выход`.
1. Закрыть окна `TessaClient` и `Winium.Desktop.Driver`.

При прохождении обучения вы всегда можете вернуться в меню выбора сценария обучения, нажав в панели управления обучением кнопку `Меню`.

[winiumdriver]: https://mytessa.ru/downloads/Winium.Desktop.Driver.2.1.1.7z
[cswebdriver]: https://www.nuget.org/packages/Syntellect.Winium.WebDriver/
[cswebelements]: https://www.nuget.org/packages/Syntellect.Winium.Elements/

