﻿using System.Reflection;
using System.Windows;

namespace UserGuide.Extensions
{
    /// <summary>
    /// Методы расширения для <see cref="SystemParameters"/>.
    /// </summary>
    internal static class SystemParametersExtensions
    {
        private static readonly PropertyInfo DpiPropertyInfo;

        /// <summary>
        /// Значение Dpi по умолчанию.
        /// </summary>
        public const double DefaultDpi = 96.0;

        static SystemParametersExtensions()
        {
            var type = typeof(SystemParameters);
            DpiPropertyInfo = type.GetProperty("Dpi", BindingFlags.Static | BindingFlags.NonPublic);
        }

        /// <summary>
        /// Получить текущий коэффициент Dpi.
        /// </summary>
        /// <returns></returns>
        public static int GetDpi()
        {
            return (int) DpiPropertyInfo.GetValue(null, null);
        }

        /// <summary>
        /// Перевести пиксели в логические дюймы с учётом Dpi.
        /// </summary>
        /// <param name="pixel">Размер в пикселях.</param>
        /// <returns>Размер в логических дюймах.</returns>
        public static double GetInches(double pixel)
        {
            int dpi = GetDpi();
            return dpi == 0 ? pixel : pixel * DefaultDpi / dpi;
        }

        /// <summary>
        /// Перевести логические дюймы в пиксели с учётом Dpi.
        /// </summary>
        /// <param name="inches">Размер в логических дюймах.</param>
        /// <returns>Размер в пикселях.</returns>
        public static double GetPixels(double inches)
        {
            int dpi = GetDpi();
            return dpi == 0 ? inches : inches * dpi / DefaultDpi;
        }
    }
}
