﻿using System.Threading;
using System.Threading.Tasks;
using UserGuide.Extensions;
using UserGuide.Windows;

namespace UserGuide.Guides
{
    /// <summary>
    /// Заключительный сценарий (в меню не отображается).
    /// </summary>
    public class ConclusionGuide : Guide
    {
        /// <inheritdoc/>
        public override string Caption => string.Empty;

        /// <inheritdoc/>
        public override int Order => 0;

        /// <inheritdoc/>
        public override GuideState State => GuideState.Conclusion;

        /// <inheritdoc/>
        public override async Task StartAsync(CancellationToken cancellationToken)
        {
            // Отобразим окно с Markdown и Emoji
            // Диалог отображается до нажатия на кнопку
            await this.Provider.ShowTooltipAsync("На этом обучение **закончено**! *Спасибо*, что прошли его **до конца**! :smiley:",
                this.Provider.MainWindow,
                x => x
                    .UseInteractionColor() // Требуется действие пользователя
                    .Place(Placement.Center) // Окно по центру
                    .UseMarkdown() // Текст в формате Markdown
                    .UseButtons(new ButtonDescriptor("До новых встреч!", 0)), // Устанавливается кнопка завершения диалога
                cancellationToken);
        }
    }
}
