﻿using System.Threading;
using System.Threading.Tasks;
using UserGuide.Extensions;
using UserGuide.Windows;

namespace UserGuide.Guides
{
    /// <summary>
    /// Сценарий вступления (в меню не отображается).
    /// </summary>
    public sealed class IntroGuide : Guide
    {
        /// <inheritdoc/>
        public override string Caption => string.Empty;

        /// <inheritdoc/>
        public override int Order => 0;

        /// <inheritdoc/>
        public override GuideState State => GuideState.Introduction;

        /// <inheritdoc/>
        public override async Task StartAsync(CancellationToken cancellationToken)
        {
            cancellationToken.ThrowIfCancellationRequested();

            // Показываем пример подсказки автоматизации.
            await this.Provider.ShowTooltipAsync("Если вы **видите такую подсказку**, пожалуйста, **не трогайте клавиатуру и мышь**.\n\nБудет выполняться эмуляция нажатия кнопок для выполнения действий пользователя.",
                this.Provider.MainWindow,
                x => x.Place(Placement.Center).CloseAfter(12).UseMarkdown(),
                cancellationToken);

            await this.Provider.DelayAsync(cancellationToken: cancellationToken);

            // Показываем пример интерактивной подсказки.
            await this.Provider.ShowTooltipAsync("Если вы **видите такую подсказку**, **выполните требующиеся от вас действия**.\n\nПожалуйста, нажмите на кнопку ниже.",
                this.Provider.MainWindow,
                x => x
                    .UseInteractionColor()
                    .Place(Placement.Center)
                    .UseMarkdown()
                    .UseButtons(new ButtonDescriptor("Всё понятно", 0)),
                cancellationToken);

            await this.Provider.DelayAsync(cancellationToken: cancellationToken);

            // Выделим главное окно
            await using (await Highlighter.ShowAsync(this.Provider.MainWindow))
            {
                // Отобразим подсказку
                await this.Provider.ShowTooltipAsync("Так выглядит выделение элементов.\nКогда есть выделение, вы не сможете взаимодействовать с целевым элементом.",
                    this.Provider.MainWindow,
                    x => x
                        .Place(Placement.Center) // Выведем окно по центру 
                        .CloseAfter(6), // Закроем его через 6 секунд
                    cancellationToken);
            }
            // Сделаем дополнительную задержку перед следующим действием
            await this.Provider.DelayAsync(cancellationToken: cancellationToken);
        }
    }
}
