﻿using System.Threading;
using System.Threading.Tasks;

namespace UserGuide
{
    /// <summary>
    /// Интерфейс сценария обучения пользователя.
    /// </summary>
    public interface IGuide
    {
        /// <summary>
        /// Краткое название сценария для его отображения в меню выбора сценариев обучения.
        /// Должно быть лаконичным и отражать суть сценария.
        /// </summary>
        string Caption { get; }

        /// <summary>
        /// Порядковый номер сценария при отображении в меню (условно, его номер в меню).
        /// Обязательно задавать уникальный номер.
        /// </summary>
        int Order { get; }

        /// <summary>
        /// Состояние сценария.
        /// Определяет, когда выполняется сценарий: при запуске приложения, по кнопке из меню и др.
        /// </summary>
        GuideState State { get; }

        /// <summary>
        /// Поставщик основных данных для обучения.
        /// Содержит полностью настроенную инфраструктуру для использования в сценариях (задержки, драйвер Winium, окно TessaClient).
        /// </summary>
        GuideProvider Provider { get; set; }

        /// <summary>
        /// Метод запуска сценария.
        /// </summary>
        /// <param name="cancellationToken">Объект для отмены асинхронной операции.</param>
        /// <returns>Задача.</returns>
        Task StartAsync(CancellationToken cancellationToken);
    }
}
