﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Threading;
using Microsoft.Extensions.Configuration;

namespace UserGuide
{
    internal class Program
    {
        private static readonly Dictionary<string, string> switchMappings =
           new()
           {
               { "-d", "DriverPath" },
               { "-c", "ClientPath" },
               { "-u", "UserName" },
               { "-p", "Password" },
           };

        static async Task Main(string[] args)
        {
            // Запустить и запомнить диспетчер.
            await DispatcherHelper.RunAsync();
            DispatcherHelper.Dispatcher.UnhandledException += Dispatcher_UnhandledException;
            try
            {
                args = args.Select(x => x.Replace(':', '=')).ToArray();
                var options = new ConfigurationBuilder()
                    .AddJsonFile("app.json", false)
                    .AddCommandLine(args, switchMappings)
                    .Build()
                    .Get<Options>();

                await new GuideProvider(options)
                    .RunAsync();
            }
            finally
            {
                // Остановить диспетчер.
                await DispatcherHelper.ShutdownAsync();
            }
        }

        private static async void Dispatcher_UnhandledException(object sender, DispatcherUnhandledExceptionEventArgs e)
        {
            if (e.Exception is not OperationCanceledException)
            {
                await GuideHelper.ShowErrorAsync(e.Exception, "Необработанная ошибка!");
            }
        }
    }
}
