﻿using System;
using System.Threading.Tasks;
using System.Windows.Controls;
using System.Windows.Media;

namespace UserGuide.Windows
{
    /// <summary>
    /// Дескриптор кнопки окна.
    /// </summary>
    /// <param name="Text">Текст кнопки, должен быть непустым.</param>
    /// <param name="Code">Код завершения, должен быть уникальным.</param>
    public record ButtonDescriptor(string Text, int Code);

    /// <summary>
    /// Настройки окна подсказки.
    /// </summary>
    public record TooltipOptions
    {
        /// <summary>
        /// Текст в формате Markdown.
        /// </summary>
        public bool MessageInMarkdown { get; init; }

        /// <summary>
        /// Режим отображения окна.
        /// </summary>
        public Mode Mode { get; init; } = Mode.AutomationTooltip;

        /// <summary>
        /// Пользовательский цвет для фона отображаемого окна с подсказкой.
        /// </summary>
        public Color? CustomColor { get; init; }

        /// <summary>
        /// Местоположение окна с подсказкой относительно элемента.
        /// </summary>
        public Placement Placement { get; init; } = Placement.Bottom;

        /// <summary>
        /// Местоположение стрелки. Учитывается только если <see cref="Placement"/> отлично от <see cref="Placement.Center"/>.
        /// </summary>
        public ArrowPlacement ArrowPlacement { get; init; } = ArrowPlacement.Center;

        /// <summary>
        /// Ориентация кнопок.
        /// </summary>
        public Orientation ButtonsOrientation { get; init; } = Orientation.Horizontal;

        /// <summary>
        /// Дескрипторы кнопок.
        /// </summary>
        public ButtonDescriptor[] ButtonDescriptors { get; init; }

        /// <summary>
        /// Максимальное время отображения окна в миллисекундах.
        /// </summary>
        public TimeSpan? ShowTime { get; init; }

        /// <summary>
        /// Задача, по завершении которой нужно закрыть окно.
        /// </summary>
        public Task CloseTask { get; init; }

        /// <summary>
        /// Закрывать ли окно при клике на нём.
        /// </summary>
        public bool CloseOnClick { get; init; }
    }
}
